@@ -449,13 +449,9 @@ int main(int argc, char* argv[])
 unsigned char ParseArg(int argc, char* argv[], struct ParstArgsData_Struct *pDestArg, unsigned char MaxDestArg )
 {
 	unsigned char ArgCount;	// Counters
-	unsigned char Status;	// Status of parsing: 0x01 value needed
 	int DestArg;	// Count of parst arguments
 	char *pValues;	// Pointer to values
 
-	// Nothing startet
-	Status	= 0;
-
 	// No args parst yet
 	DestArg	= -1;
 
@@ -801,7 +797,7 @@ int RunUnInstaller( const struct RunningOptionData_Struct *pRunData )
 			break;
 
 			default:
-				_snprintf( g_ErrorMessage, SQLINST_MAX_ERROR_BUFFER, "Internel error: %s: Can not run: %d\n", __FUNCTION__, pRunData->DoFollow );
+				_snprintf( g_ErrorMessage, SQLINST_MAX_ERROR_BUFFER, "Internel error: %s: Can not run: %d\n", "RunUnInstaller", pRunData->DoFollow );
 				g_ErrorMessage[SQLINST_MAX_ERROR_BUFFER-1] = 0;
 				MessageToUser( g_ErrorMessage );
 				// Break with error
