@@ -1,4 +1,9 @@
-/*
+/**
+ * @file impexp.c
+ * SQLite extension module for importing/exporting
+ * database information from/to SQL source text and
+ * export to CSV text.
+ *
  * 2007 January 27
  *
  * The author disclaims copyright to this source code.  In place of
@@ -10,17 +15,15 @@
  *
  ********************************************************************
  *
- * SQLite extension module for importing/exporting
- * database information from/to SQL source text and
- * export to CSV text.
- *
+ * <pre>
  * Usage:
  *
  *  SQLite function:
  *       SELECT import_sql(filename);
  *
  *  C function:
- *       int impexp_import_sql(sqlite3 *db, char *filename);
+ *       int impexp_import_sql(sqlite3 *db,
+ *                             char *filename);
  *
  *       Reads SQL commands from filename and executes them
  *       against the current database. Returns the number
@@ -121,22 +124,22 @@
  *          SELECT export_xml('out.xml', 0, 2, 'TBL_A', 'ROW', 'A');
  *          -- XML output
  *            <TBL_A>
- *             <ROW>
- *              <a TYPE="INTEGER">1</a>
- *              <b TYPE="REAL">2.1</b>
- *             </ROW>
- *             <ROW>
- *              <a TYPE="INTEGER">3</a>
- *              <b TYPE="TEXT">foo</b>
- *             </ROW>
- *             <ROW>
- *              <a TYPE="TEXT"></a>
- *              <b TYPE="NULL"></b>
- *             </ROW>
- *             <ROW>
- *              <a TYPE="BLOB">&#x01;&#x02;&x03;</a>
- *              <b TYPE="TEXT">&lt;blob&gt;</b>
- *             </ROW>
+ *              <ROW>
+ *                &lt;a TYPE="INTEGER"&gt;1&lt;/a&gt;
+ *                &lt;b TYPE="REAL"&gt;2.1&lt;/b&gt;
+ *              </ROW>
+ *              <ROW>
+ *                &lt;a TYPE="INTEGER"&gt;3&lt;/a&gt;
+ *                &lt;b TYPE="TEXT"&gt;foo&lt;/b&gt;
+ *              </ROW>
+ *              <ROW>
+ *                &lt;a TYPE="TEXT"&gt;&lt;/a&gt;
+ *                &lt;b TYPE="NULL"&gt;&lt;/b&gt;
+ *              </ROW>
+ *              <ROW>
+ *                &lt;a TYPE="BLOB"&gt;&#x01;&#x02;&x03;&lt;/a&gt;
+ *                &lt;b TYPE="TEXT"&gt;&amp;lt;blob&amp;gt;&lt;/b&gt;
+ *              </ROW>
  *            </TBL_A>
  *
  *       Quoting of XML entities is performed only on the data,
@@ -174,6 +177,7 @@
  *
  * On Win32 the filename argument may be specified as NULL in order
  * to open a system file dialog for interactive filename selection.
+ * </pre>
  */
 
 #ifdef STANDALONE
@@ -181,7 +185,7 @@
 #define sqlite3_api_routines void
 #else
 #include <sqlite3ext.h>
-SQLITE_EXTENSION_INIT1
+static SQLITE_EXTENSION_INIT1
 #endif
 
 #include <stdlib.h>
@@ -199,16 +203,27 @@ SQLITE_EXTENSION_INIT1
 
 #include "impexp.h"
 
-/* JSON output helper */
+/**
+ * @typedef struct json_pfs
+ * @struct json_pfs
+ * JSON output helper structure
+ */
 
 typedef struct {
-    impexp_putc pfunc;
-    void *parg;
+    impexp_putc pfunc;	/**< function like fputc() */
+    void *parg;		/**< argument to function */
 } json_pfs;
 
 static const char space_chars[] = " \f\n\r\t\v";
 
-#define ISSPACE(c) ((c) && strchr(space_chars, (c)) != 0)
+#define ISSPACE(c) ((c) && (strchr(space_chars, (c)) != 0))
+
+/**
+ * Read one line of input into dynamically allocated buffer
+ * which the caller must free with sqlite3_free()
+ * @param fin FILE pointer
+ * @result dynamically allocated input line
+ */
 
 static char *
 one_input_line(FILE *fin)
@@ -247,7 +262,7 @@ one_input_line(FILE *fin)
 	while (line[n]) {
 	    n++;
 	}
-	if (n > 0 && line[n-1] == '\n') {
+	if ((n > 0) && (line[n-1] == '\n')) {
 	    n--;
 	    line[n] = 0;
 	    eol = 1;
@@ -256,20 +271,32 @@ one_input_line(FILE *fin)
     tmp = sqlite3_realloc(line, n + 1);
     if (!tmp) {
 	sqlite3_free(line);
-	return 0;
     }
     return tmp;
 }
 
+/**
+ * Test if string ends with a semicolon
+ * @param str string to be tested
+ * @param n length of string
+ * @result true or false
+ */
+
 static int
 ends_with_semicolon(const char *str, int n)
 {
-    while (n > 0 && ISSPACE(str[n - 1])) {
+    while ((n > 0) && ISSPACE(str[n - 1])) {
 	n--;
     }
-    return n > 0 && str[n - 1] == ';';
+    return (n > 0) && (str[n - 1] == ';');
 }
 
+/**
+ * Test if string contains entirely whitespace or SQL comment
+ * @param str string to be tested
+ * @result true or false
+ */
+
 static int
 all_whitespace(const char *str)
 {
@@ -277,9 +304,9 @@ all_whitespace(const char *str)
 	if (ISSPACE(str[0])) {
 	    continue;
 	}
-	if (str[0] == '/' && str[1] == '*') {
+	if ((str[0] == '/') && (str[1] == '*')) {
 	    str += 2;
-	    while (str[0] && (str[0] != '*' || str[1] != '/')) {
+	    while (str[0] && ((str[0] != '*') || (str[1] != '/'))) {
 		str++;
 	    }
 	    if (!str[0]) {
@@ -288,9 +315,9 @@ all_whitespace(const char *str)
 	    str++;
 	    continue;
 	}
-	if (str[0] == '-' && str[1] == '-') {
+	if ((str[0] == '-') && (str[1] == '-')) {
 	    str += 2;
-	    while (str[0] && str[0] != '\n') {
+	    while (str[0] && (str[0] != '\n')) {
 		str++;
 	    }
 	    if (!str[0]) {
@@ -303,6 +330,13 @@ all_whitespace(const char *str)
     return 1;
 }
 
+/**
+ * Process contents of FILE pointer as SQL commands
+ * @param db SQLite database to work on
+ * @param fin input FILE pointer
+ * @result number of errors
+ */
+
 static int
 process_input(sqlite3 *db, FILE *fin)
 {
@@ -370,6 +404,18 @@ process_input(sqlite3 *db, FILE *fin)
     return errors;
 }
 
+/**
+ * SQLite function to quote SQLite value depending on optional quote mode
+ * @param context SQLite function context
+ * @param argc number of arguments
+ * @param argv argument vector
+ *
+ * Layout of arguments:
+ *
+ * argv[0] - value to be quoted<br>
+ * argv[1] - value of quote mode (optional)<br>
+ */
+
 static void
 quote_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
@@ -393,7 +439,7 @@ quote_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
     case SQLITE_BLOB: {
 	char *text = 0;
-	char const *blob = sqlite3_value_blob(argv[0]);
+	unsigned char *blob = (unsigned char *) sqlite3_value_blob(argv[0]);
 	int nblob = sqlite3_value_bytes(argv[0]);
 
 	if (2 * nblob + 4 > 1000000000) {
@@ -465,6 +511,7 @@ quote_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 	p = sqlite3_malloc(i + n + 3);
 	if (!p) {
 	    sqlite3_result_error(context, "out of memory", -1);
+	    return;
 	}
 	p[0] = '\'';
 	for (i = 0, n = 1; arg[i]; i++) {
@@ -482,6 +529,13 @@ quote_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
 }
 
+/**
+ * SQLite function to quote an SQLite value in CSV format
+ * @param context SQLite function context
+ * @param argc number of arguments
+ * @param argv argument vector
+ */
+
 static void
 quote_csv_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
@@ -500,7 +554,7 @@ quote_csv_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
     case SQLITE_BLOB: {
 	char *text = 0;
-	char const *blob = sqlite3_value_blob(argv[0]);
+	unsigned char *blob = (unsigned char *) sqlite3_value_blob(argv[0]);
 	int nblob = sqlite3_value_bytes(argv[0]);
 
 	if (2 * nblob + 4 > 1000000000) {
@@ -546,6 +600,7 @@ quote_csv_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 	p = sqlite3_malloc(i + n + 3);
 	if (!p) {
 	    sqlite3_result_error(context, "out of memory", -1);
+	    return;
 	}
 	p[0] = '"';
 	for (i = 0, n = 1; arg[i]; i++) {
@@ -563,6 +618,13 @@ quote_csv_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
 }
 
+/**
+ * SQLite function to make XML indentation
+ * @param context SQLite function context
+ * @param argc number of arguments
+ * @param argv argument vector
+ */
+
 static void
 indent_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
@@ -580,6 +642,13 @@ indent_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     sqlite3_result_text(context, spaces, n, SQLITE_STATIC);
 }
 
+/**
+ * SQLite function to quote a string for XML
+ * @param context SQLite function context
+ * @param argc number of arguments
+ * @param argv argument vector
+ */
+
 static void
 quote_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
@@ -626,7 +695,7 @@ quote_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
     case SQLITE_BLOB: {
 	char *text = 0;
-	char const *blob = sqlite3_value_blob(argv[0]);
+	unsigned char *blob = (unsigned char *) sqlite3_value_blob(argv[0]);
 	int nblob = sqlite3_value_bytes(argv[0]);
 	int i, k = 0;
 
@@ -665,9 +734,9 @@ quote_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 	    return;
 	}
 	for (i = 0, n = 0; arg[i]; i++) {
-	    if (arg[i] == '"' || arg[i] == '\'' ||
-		arg[i] == '<' || arg[i] == '>' ||
-		arg[i] == '&' || arg[i] < ' ') {
+	    if ((arg[i] == '"') || (arg[i] == '\'') ||
+		(arg[i] == '<') || (arg[i] == '>') ||
+		(arg[i] == '&') || (arg[i] < ' ')) {
 		n += 5;
 	    }
 	}
@@ -723,7 +792,7 @@ quote_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
 		p[n++] = xdigits[(arg[i] >> 4 ) & 0x0F];
 		p[n++] = xdigits[arg[i] & 0x0F];
 		p[n++] = ';';
-	    } else if (addtype < 0 && arg[i] == ' ') {
+	    } else if (addtype < 0 && (arg[i] == ' ')) {
 		p[n++] = '&';
 		p[n++] = '#';
 		p[n++] = 'x';
@@ -742,6 +811,13 @@ quote_xml_func(sqlite3_context *context, int argc, sqlite3_value **argv)
     }
 }
 
+/**
+ * SQLite function to read and process SQL commands from a file
+ * @param ctx SQLite function context
+ * @param nargs number of arguments
+ * @param args argument vector
+ */
+
 static void
 import_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 {
@@ -787,6 +863,8 @@ done:
     sqlite3_result_int(ctx, sqlite3_changes(db) - changes0);
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_import_sql(sqlite3 *db, char *filename)
 {
@@ -829,16 +907,27 @@ done:
     return sqlite3_changes(db) - changes0;
 }
 
+/**
+ * @typedef DUMP_DATA
+ * @struct DUMP_DATA
+ * Structure for dump callback
+ */
+
 typedef struct {
-    sqlite3 *db;
-    int with_schema;
-    int quote_mode;
-    char *where;
-    int nlines;
-    int indent;
-    FILE *out;
+    sqlite3 *db;	/**< SQLite database pointer */
+    int with_schema;	/**< if true, output schema */
+    int quote_mode;	/**< mode for quoting data */
+    char *where;	/**< optional where clause of dump */
+    int nlines;		/**< counter for output lines */
+    int indent;		/**< current indent level */
+    FILE *out;		/**< output file pointer */
 } DUMP_DATA;
 
+/**
+ * Write indentation to dump
+ * @param dd information structure for dump
+ */
+
 static void
 indent(DUMP_DATA *dd)
 {
@@ -849,10 +938,20 @@ indent(DUMP_DATA *dd)
     }
 }
 
+/**
+ * Execute SQL to dump contents of one table
+ * @param dd information structure for dump
+ * @param errp pointer receiving error message
+ * @param fmt if true, use sqlite3_*printf() on SQL
+ * @param query SQL text to perform dump of table
+ * @param ... optional arguments
+ * @result SQLite error code
+ */
+
 static int
 table_dump(DUMP_DATA *dd, char **errp, int fmt, const char *query, ...)
 {
-    sqlite3_stmt *select;
+    sqlite3_stmt *select = 0;
     int rc;
     const char *rest, *q = query;
     va_list ap;
@@ -877,7 +976,7 @@ table_dump(DUMP_DATA *dd, char **errp, int fmt, const char *query, ...)
     if (fmt) {
 	sqlite3_free((char *) q);
     }
-    if (rc != SQLITE_OK || !select) {
+    if ((rc != SQLITE_OK) || !select) {
 	return rc;
     }
     rc = sqlite3_step(select);
@@ -905,15 +1004,49 @@ table_dump(DUMP_DATA *dd, char **errp, int fmt, const char *query, ...)
     return rc;
 }
 
-static char *
-append(char *in, char const *append, char quote)
-{
-    int len, i;
-    int nappend = append ? strlen(append) : 0;
-    int nin = in ? strlen(in) : 0;
-    char *tmp;
+/**
+ * Free dynamically allocated string buffer
+ * @param in input string pointer
+ */
 
-    len = nappend + nin + 1;
+static void
+append_free(char **in)
+{
+    long *p = (long *) *in;
+
+    if (p) {
+	p -= 2;
+	sqlite3_free(p);
+	*in = 0;
+    }
+}
+
+/**
+ * Append a string to dynamically allocated string buffer
+ * with optional quoting
+ * @param in input string pointer
+ * @param append string to append
+ * @param quote quote character or NUL
+ * @result new string to be free'd with append_free()
+ */
+
+static char *
+append(char **in, char const *append, char quote)
+{
+    long *p = (long *) *in;
+    long len, maxlen, actlen;
+    int i;
+    char *pp;
+    int nappend = append ? strlen(append) : 0;
+
+    if (p) {
+	p -= 2;
+	maxlen = p[0];
+	actlen = p[1];
+    } else {
+	maxlen = actlen = 0;
+    }
+    len = nappend + actlen;
     if (quote) {
 	len += 2;
 	for (i = 0; i < nappend; i++) {
@@ -922,35 +1055,51 @@ append(char *in, char const *append, char quote)
 	    }
 	}
     } else if (!nappend) {
-	return in;
+	return *in;
     }
-    tmp = (char *) sqlite3_realloc(in, len);
-    if (!tmp) {
-	sqlite3_free(in);
-	return 0;
-    }
-    in = tmp;
-    if (quote) {
-	char *p = in + nin;
+    if (len >= maxlen - 1) {
+	long *q;
 
-	*p++ = quote;
+	maxlen = (len + 0x03ff) & (~0x3ff);
+	q = (long *) sqlite3_realloc(p, maxlen + 1 + 2 * sizeof (long));
+	if (!q) {
+	    return 0;
+	}
+	if (!p) {
+	    q[1] = 0;
+	}
+	p = q;
+	p[0] = maxlen;
+	*in = (char *) (p + 2);
+    }
+    pp = *in + actlen;
+    if (quote) {
+	*pp++ = quote;
 	for (i = 0; i < nappend; i++) {
-	    *p++ = append[i];
+	    *pp++ = append[i];
 	    if (append[i] == quote) {
-		*p++ = quote;
+		*pp++ = quote;
 	    }
 	}
-	*p++ = quote;
-	*p++ = '\0';
+	*pp++ = quote;
+	*pp = '\0';
     } else {
 	if (nappend) {
-	    memcpy(in + nin, append, nappend);
+	    memcpy(pp, append, nappend);
+	    pp += nappend;
+	    *pp = '\0';
 	}
-	in[len - 1] = '\0';
     }
-    return in;
+    p[1] = pp - *in;
+    return *in;
 }
 
+/**
+ * Quote string for XML output during dump
+ * @param dd information structure for dump
+ * @param str string to be output
+ */
+
 static void
 quote_xml_str(DUMP_DATA *dd, char *str)
 {
@@ -989,6 +1138,15 @@ quote_xml_str(DUMP_DATA *dd, char *str)
     }
 }
 
+/**
+ * Callback for sqlite3_exec() to dump one data row
+ * @param udata information structure for dump
+ * @param nargs number of columns
+ * @param args column data
+ * @param cols column labels
+ * @result 0 to continue, 1 to abort
+ */
+
 static int
 dump_cb(void *udata, int nargs, char **args, char **cols)
 {
@@ -996,7 +1154,7 @@ dump_cb(void *udata, int nargs, char **args, char **cols)
     const char *table, *type, *sql;
     DUMP_DATA *dd = (DUMP_DATA *) udata;
 
-    if (nargs != 3 || args == NULL) {
+    if ((nargs != 3) || (args == NULL)) {
 	return 1;
     }
     table = args[0];
@@ -1021,65 +1179,61 @@ dump_cb(void *udata, int nargs, char **args, char **cols)
 	    sqlite3_stmt *stmt = 0;
 	    char *creat = 0, *table_info = 0;
    
-	    table_info = append(table_info, "PRAGMA table_info(", 0);
-	    table_info = append(table_info, table, '"');
-	    table_info = append(table_info, ");", 0);
+	    append(&table_info, "PRAGMA table_info(", 0);
+	    append(&table_info, table, '"');
+	    append(&table_info, ")", 0);
 #if defined(HAVE_SQLITE3PREPAREV2) && HAVE_SQLITE3PREPAREV2
 	    rc = sqlite3_prepare_v2(dd->db, table_info, -1, &stmt, 0);
 #else
 	    rc = sqlite3_prepare(dd->db, table_info, -1, &stmt, 0);
 #endif
-	    if (table_info) {
-		sqlite3_free(table_info);
-		table_info = 0;
-	    }
-	    if (rc != SQLITE_OK || !stmt) {
+	    append_free(&table_info);
+	    if ((rc != SQLITE_OK) || !stmt) {
 bailout0:
-		if (creat) {
-		    sqlite3_free(creat);
+		if (stmt) {
+		    sqlite3_finalize(stmt);
 		}
+		append_free(&creat);
 		return 1;
 	    }
-	    creat = append(creat, table, '"');
-	    creat = append(creat, "(", 0);
+	    append(&creat, table, '"');
+	    append(&creat, "(", 0);
 	    rc = sqlite3_step(stmt);
 	    while (rc == SQLITE_ROW) {
 		const char *p;
 
 		p = (const char *) sqlite3_column_text(stmt, 1);
-		creat = append(creat, p, '"');
-		creat = append(creat, " ", 0);
+		append(&creat, p, '"');
+		append(&creat, " ", 0);
 		p = (const char *) sqlite3_column_text(stmt, 2);
 		if (p && p[0]) {
-		    creat = append(creat, p, 0);
+		    append(&creat, p, 0);
 		}
 		if (sqlite3_column_int(stmt, 5)) {
-		    creat = append(creat, " PRIMARY KEY", 0);
+		    append(&creat, " PRIMARY KEY", 0);
 		}
 		if (sqlite3_column_int(stmt, 3)) {
-		    creat = append(creat, " NOT NULL", 0);
+		    append(&creat, " NOT NULL", 0);
 		}
 		p = (const char *) sqlite3_column_text(stmt, 4);
 		if (p && p[0]) {
-		    creat = append(creat, " DEFAULT ", 0);
-		    creat = append(creat, p, 0);
+		    append(&creat, " DEFAULT ", 0);
+		    append(&creat, p, 0);
 		}
 		rc = sqlite3_step(stmt);
 		if (rc == SQLITE_ROW) {
-		    creat = append(creat, ",", 0);
+		    append(&creat, ",", 0);
 		}
 	    }
-	    rc = sqlite3_finalize(stmt);
-	    if (rc != SQLITE_OK) {
+	    if (rc != SQLITE_DONE) {
 		goto bailout0;
 	    }
-	    creat = append(creat, ")", 0);
+	    sqlite3_finalize(stmt);
+	    append(&creat, ")", 0);
 	    if (creat && fprintf(dd->out, "CREATE TABLE %s;\n", creat) > 0) {
 		dd->nlines++;
 	    }
-	    if (creat) {
-		sqlite3_free(creat);
-	    }
+	    append_free(&creat);
 	}
     } else {
 	if (dd->with_schema) {
@@ -1088,113 +1242,111 @@ bailout0:
 	    }
 	}
     }
-    if (strcmp(type, "table") == 0 ||
-	(dd->quote_mode < 0 && strcmp(type, "view") == 0)) {
+    if ((strcmp(type, "table") == 0) ||
+	((dd->quote_mode < 0) && (strcmp(type, "view") == 0))) {
 	sqlite3_stmt *stmt = 0;
-	char *select = 0, *hdr = 0, *table_info = 0, *tmp = 0;
+	char *select = 0, *hdr = 0, *table_info = 0;
 	char buffer[256];
    
-	table_info = append(table_info, "PRAGMA table_info(", 0);
-	table_info = append(table_info, table, '"');
-	table_info = append(table_info, ");", 0);
+	append(&table_info, "PRAGMA table_info(", 0);
+	append(&table_info, table, '"');
+	append(&table_info, ")", 0);
 #if defined(HAVE_SQLITE3PREPAREV2) && HAVE_SQLITE3PREPAREV2
 	rc = sqlite3_prepare_v2(dd->db, table_info, -1, &stmt, 0);
 #else
 	rc = sqlite3_prepare(dd->db, table_info, -1, &stmt, 0);
 #endif
-	if (rc != SQLITE_OK || !stmt) {
+	append_free(&table_info);
+	if ((rc != SQLITE_OK) || !stmt) {
 bailout1:
-	    if (hdr) {
-		sqlite3_free(hdr);
-	    }
-	    if (select) {
-		sqlite3_free(select);
-	    }
-	    if (table_info) {
-		sqlite3_free(table_info);
+	    if (stmt) {
+		sqlite3_finalize(stmt);
 	    }
+	    append_free(&hdr);
+	    append_free(&select);
 	    return 1;
 	}
 	if (dd->quote_mode < -1) {
 	    if (dd->where) {
-		select = append(select, "SELECT ", 0);
+		append(&select, "SELECT ", 0);
 		sprintf(buffer, "indent_xml(%d)", dd->indent);
-		select = append(select, buffer, 0);
-		select = append(select, " || '<' || quote_xml(", 0);
-		select = append(select, dd->where, '"');
-		select = append(select, ",-1) || '>\n' || ", 0);
+		append(&select, buffer, 0);
+		append(&select, " || '<' || quote_xml(", 0);
+		append(&select, dd->where, '"');
+		append(&select, ",-1) || '>\n' || ", 0);
 	    } else {
-		select = append(select, "SELECT ", 0);
+		append(&select, "SELECT ", 0);
 	    }
 	} else if (dd->quote_mode < 0) {
 	    if (dd->where) {
-		select = append(select, "SELECT quote_csv(", 0);
-		select = append(select, dd->where, '"');
-		select = append(select, ") || ',' || ", 0);
+		append(&select, "SELECT quote_csv(", 0);
+		append(&select, dd->where, '"');
+		append(&select, ") || ',' || ", 0);
 	    } else {
-		select = append(select, "SELECT ", 0);
+		append(&select, "SELECT ", 0);
 	    }
 	    if (dd->indent) {
-		hdr = append(hdr, select, 0);
+		append(&hdr, select, 0);
 	    }
 	} else {
+	    char *tmp = 0;
+
 	    if (dd->with_schema) {
-		select = append(select, "SELECT 'INSERT INTO ' || ", 0);
+		append(&select, "SELECT 'INSERT INTO ' || ", 0);
 	    } else {
-		select = append(select, "SELECT 'INSERT OR REPLACE INTO ' || ",
-				0);
+		append(&select, "SELECT 'INSERT OR REPLACE INTO ' || ", 0);
 	    }
-	    tmp = append(tmp, table, '"');
+	    append(&tmp, table, '"');
 	    if (tmp) {
-		select = append(select, tmp, '\'');
-		sqlite3_free(tmp);
-		tmp = 0;
+		append(&select, tmp, '\'');
+		append_free(&tmp);
 	    }
 	}
-	if (dd->quote_mode >= 0 && !dd->with_schema) {
-	    select = append(select, " || ' (' || ", 0);
+	if ((dd->quote_mode >= 0) && !dd->with_schema) {
+	    char *tmp = 0;
+
+	    append(&select, " || ' (' || ", 0);
 	    rc = sqlite3_step(stmt);
 	    while (rc == SQLITE_ROW) {
 		const char *text = (const char *) sqlite3_column_text(stmt, 1);
 
-		tmp = append(tmp, text, '"');
+		append(&tmp, text, '"');
 		if (tmp) {
-		    select = append(select, tmp, '\'');
-		    sqlite3_free(tmp);
-		    tmp = 0;
+		    append(&select, tmp, '\'');
+		    append_free(&tmp);
 		}
 		rc = sqlite3_step(stmt);
 		if (rc == SQLITE_ROW) {
-		    select = append(select, " || ',' || ", 0);
+		    append(&select, " || ',' || ", 0);
 		}
 	    }
-	    rc = sqlite3_reset(stmt);
-	    if (rc != SQLITE_OK) {
+	    if (rc != SQLITE_DONE) {
 		goto bailout1;
 	    }
-	    select = append(select, "|| ')'", 0);
+	    sqlite3_reset(stmt);
+	    append(&select, "|| ')'", 0);
 	}
-	if (dd->quote_mode == -1 && dd->indent) {
+	if ((dd->quote_mode == -1) && dd->indent) {
 	    rc = sqlite3_step(stmt);
 	    while (rc == SQLITE_ROW) {
 		const char *text = (const char *) sqlite3_column_text(stmt, 1);
 
-		hdr = append(hdr, "quote_csv(", 0);
-		hdr = append(hdr, text, '"');
+		append(&hdr, "quote_csv(", 0);
+		append(&hdr, text, '"');
 		rc = sqlite3_step(stmt);
 		if (rc == SQLITE_ROW) {
-		    hdr = append(hdr, ") || ',' || ", 0);
+		    append(&hdr, ") || ',' || ", 0);
 		} else {
-		    hdr = append(hdr, ")", 0);
+		    append(&hdr, ")", 0);
 		}
 	    }
-	    rc = sqlite3_reset(stmt);
-	    if (rc != SQLITE_OK) {
+	    if (rc != SQLITE_DONE) {
 		goto bailout1;
 	    }
+	    sqlite3_reset(stmt);
 	}
 	if (dd->quote_mode >= 0) {
-	    select = append(select, " || ' VALUES(' || ", 0);
+	    append(&select, " || ' VALUES(' || ", 0);
 	}
 	rc = sqlite3_step(stmt);
 	while (rc == SQLITE_ROW) {
@@ -1204,18 +1356,18 @@ bailout1:
 
 	    if (dd->quote_mode < -1) {
 		sprintf(buffer, "indent_xml(%d)", dd->indent + 1);
-		select = append(select, buffer, 0);
-		select = append(select, "|| '<' || quote_xml(", 0);
-		select = append(select, text, '\'');
-		select = append(select, ",-1) || quote_xml(", 0);
-		select = append(select, text, '"');
-		select = append(select, ",1) || '</' || quote_xml(", 0);
-		select = append(select, text, '\'');
-		select = append(select, ",-1) || '>\n'", 0);
+		append(&select, buffer, 0);
+		append(&select, "|| '<' || quote_xml(", 0);
+		append(&select, text, '\'');
+		append(&select, ",-1) || quote_xml(", 0);
+		append(&select, text, '"');
+		append(&select, ",1) || '</' || quote_xml(", 0);
+		append(&select, text, '\'');
+		append(&select, ",-1) || '>\n'", 0);
 	    } else if (dd->quote_mode < 0) {
 		/* leave out BLOB columns */
-		if ((tlen >= 4 && strncasecmp(type, "BLOB", 4) == 0) ||
-		    (tlen >= 6 && strncasecmp(type, "BINARY", 6) == 0)) {
+		if (((tlen >= 4) && (strncasecmp(type, "BLOB", 4) == 0)) ||
+		    ((tlen >= 6) && (strncasecmp(type, "BINARY", 6) == 0))) {
 		    rc = sqlite3_step(stmt);
 		    if (rc != SQLITE_ROW) {
 			tlen = strlen(select);
@@ -1225,77 +1377,80 @@ bailout1:
 		    }
 		    continue;
 		}
-		select = append(select, "quote_csv(", 0);
-		select = append(select, text, '"');
+		append(&select, "quote_csv(", 0);
+		append(&select, text, '"');
 	    } else {
-		select = append(select, "quote_sql(", 0);
-		select = append(select, text, '"');
+		append(&select, "quote_sql(", 0);
+		append(&select, text, '"');
 		if (dd->quote_mode) {
 		    char mbuf[32];
 
 		    sprintf(mbuf, ",%d", dd->quote_mode);
-		    select = append(select, mbuf, 0);
+		    append(&select, mbuf, 0);
 		}
 	    }
 	    rc = sqlite3_step(stmt);
 	    if (rc == SQLITE_ROW) {
 		if (dd->quote_mode >= -1) {
-		    select = append(select, ") || ',' || ", 0);
+		    append(&select, ") || ',' || ", 0);
 		} else {
-		    select = append(select, " || ", 0);
+		    append(&select, " || ", 0);
 		}
 	    } else {
 		if (dd->quote_mode >= -1) {
-		    select = append(select, ") ", 0);
+		    append(&select, ") ", 0);
 		} else {
-		    select = append(select, " ", 0);
+		    append(&select, " ", 0);
 		}
 	    }
 	}
-	rc = sqlite3_finalize(stmt);
-	if (rc != SQLITE_OK) {
+	if (rc != SQLITE_DONE) {
 	    goto bailout1;
 	}
+	sqlite3_finalize(stmt);
+	stmt = 0;
 	if (dd->quote_mode >= 0) {
-	    select = append(select, "|| ')' FROM ", 0);
+	    append(&select, "|| ')' FROM ", 0);
 	} else {
-	    if (dd->quote_mode < -1 && dd->where) {
+	    if ((dd->quote_mode < -1) && dd->where) {
 		sprintf(buffer, " || indent_xml(%d)", dd->indent);
-		select = append(select, buffer, 0);
-		select = append(select, " || '</' || quote_xml(", 0);
-		select = append(select, dd->where, '"');
-		select = append(select, ",-1) || '>\n' FROM ", 0);
+		append(&select, buffer, 0);
+		append(&select, " || '</' || quote_xml(", 0);
+		append(&select, dd->where, '"');
+		append(&select, ",-1) || '>\n' FROM ", 0);
 	    } else {
-		select = append(select, "FROM ", 0);
+		append(&select, "FROM ", 0);
 	    }
 	}
-	select = append(select, table, '"');
-	if (dd->quote_mode >= 0 && dd->where) {
-	    select = append(select, " ", 0);
-	    select = append(select, dd->where, 0);
-	}
-	if (table_info) {
-	    sqlite3_free(table_info);
-	    table_info = 0;
+	append(&select, table, '"');
+	if ((dd->quote_mode >= 0) && dd->where) {
+	    append(&select, " ", 0);
+	    append(&select, dd->where, 0);
 	}
 	if (hdr) {
 	    rc = table_dump(dd, 0, 0, hdr);
-	    sqlite3_free(hdr);
+	    append_free(&hdr);
 	    hdr = 0;
 	}
 	rc = table_dump(dd, 0, 0, select);
 	if (rc == SQLITE_CORRUPT) {
-	    select = append(select, " ORDER BY rowid DESC", 0);
+	    append(&select, " ORDER BY rowid DESC", 0);
 	    rc = table_dump(dd, 0, 0, select);
 	}
-	if (select) {
-	    sqlite3_free(select);
-	    select = 0;
-	}
+	append_free(&select);
     }
     return 0;
 }
 
+/**
+ * Execute SQL on sqlite_master table in order to dump data.
+ * @param dd information structure for dump
+ * @param errp pointer receiving error message
+ * @param query SQL for sqlite3_*printf()
+ * @param ... argument list
+ * @result SQLite error code
+ */
+
 static int
 schema_dump(DUMP_DATA *dd, char **errp, const char *query, ...)
 {
@@ -1333,6 +1488,13 @@ schema_dump(DUMP_DATA *dd, char **errp, const char *query, ...)
     return rc;
 }
 
+/**
+ * SQLite function for SQL output, see impexp_export_sql
+ * @param ctx SQLite function context
+ * @param nargs number of arguments
+ * @param args argument vector
+ */
+
 static void
 export_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 {
@@ -1397,7 +1559,7 @@ export_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
     } else {
 	for (i = 2; i < nargs; i += (mode & 2) ? 2 : 1) {
 	    dd->where = 0;
-	    if ((mode & 2) && i + 1 < nargs) {
+	    if ((mode & 2) && (i + 1 < nargs)) {
 		dd->where = (char *) sqlite3_value_text(args[i + 1]);
 	    }
 	    schema_dump(dd, 0,
@@ -1423,6 +1585,13 @@ done:
     sqlite3_result_int(ctx, dd->nlines);
 }
 
+/**
+ * SQLite function for CSV output, see impexp_export_csv
+ * @param ctx SQLite function context
+ * @param nargs number of arguments
+ * @param args argument vector
+ */
+
 static void
 export_csv_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 {
@@ -1493,7 +1662,7 @@ export_csv_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 	if (sqlite3_value_type(args[i + 2]) != SQLITE_NULL) {
 	    schema = (char *) sqlite3_value_text(args[i + 2]);
 	}
-	if (!schema || schema[0] == '\0') {
+	if (!schema || (schema[0] == '\0')) {
 	    schema = "sqlite_master";
 	}
 	sql = sqlite3_mprintf("SELECT name, type, sql FROM %s"
@@ -1510,6 +1679,13 @@ done:
     sqlite3_result_int(ctx, dd->nlines);
 }
 
+/**
+ * SQLite function for XML output, see impexp_export_xml
+ * @param ctx SQLite function context
+ * @param nargs number of arguments
+ * @param args argument vector
+ */
+
 static void
 export_xml_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 {
@@ -1598,7 +1774,7 @@ export_xml_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 	if (sqlite3_value_type(args[i + 3]) != SQLITE_NULL) {
 	    schema = (char *) sqlite3_value_text(args[i + 3]);
 	}
-	if (!schema || schema[0] == '\0') {
+	if (!schema || (schema[0] == '\0')) {
 	    schema = "sqlite_master";
 	}
 	sql = sqlite3_mprintf("SELECT name, type, sql FROM %s"
@@ -1622,6 +1798,8 @@ done:
     sqlite3_result_int(ctx, dd->nlines);
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_export_sql(sqlite3 *db, char *filename, int mode, ...)
 {
@@ -1706,6 +1884,8 @@ done:
     return dd->nlines;
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_export_csv(sqlite3 *db, char *filename, int hdr, ...)
 {
@@ -1748,7 +1928,7 @@ impexp_export_csv(sqlite3 *db, char *filename, int hdr, ...)
 #ifdef _WIN32
     dd->out = fopen(filename, "wb");
 #else
-    if (hdr < 0 && access(filename, W_OK) == 0) {
+    if ((hdr < 0) && access(filename, W_OK) == 0) {
 	dd->out = fopen(filename, "a");
 	dd->indent = 0;
     } else {
@@ -1767,7 +1947,7 @@ impexp_export_csv(sqlite3 *db, char *filename, int hdr, ...)
 	char *sql;
 
 	dd->where = (prefix && prefix[0]) ? prefix : 0;
-	if (!schema || schema[0] == '\0') {
+	if (!schema || (schema[0] == '\0')) {
 	    schema = "sqlite_master";
 	}
 	sql = sqlite3_mprintf("SELECT name, type, sql FROM %s"
@@ -1788,6 +1968,8 @@ done:
     return dd->nlines;
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_export_xml(sqlite3 *db, char *filename, int append, int indnt,
 		  char *root, char *item, char *tablename, char *schema)
@@ -1804,7 +1986,7 @@ impexp_export_xml(sqlite3 *db, char *filename, int append, int indnt,
     dd->db = db;
     dd->where = item;
     dd->nlines = -1;
-    dd->indent = indnt > 0 ? indnt : 0;
+    dd->indent = (indnt > 0) ? indnt : 0;
     dd->with_schema = 0;
     dd->quote_mode = -2;
 #ifdef _WIN32
@@ -1838,7 +2020,7 @@ impexp_export_xml(sqlite3 *db, char *filename, int append, int indnt,
 	quote_xml_str(dd, root);
 	fputs(">\n", dd->out);
     }
-    if (!schema || schema[0] == '\0') {
+    if (!schema || (schema[0] == '\0')) {
 	schema = "sqlite_master";
     }
     sql = sqlite3_mprintf("SELECT name, type, sql FROM %s"
@@ -1861,6 +2043,12 @@ done:
     return dd->nlines;
 }
 
+/**
+ * Write string using JSON output function
+ * @param string string to be written
+ * @param pfs JSON output function
+ */
+
 static void
 json_pstr(const char *string, json_pfs *pfs)
 {
@@ -1870,6 +2058,12 @@ json_pstr(const char *string, json_pfs *pfs)
     }
 }
 
+/**
+ * Quote and write string using JSON output function
+ * @param string string to be written
+ * @param pfs JSON output function
+ */
+
 static void
 json_pstrq(const char *string, json_pfs *pfs)
 {
@@ -1910,7 +2104,7 @@ json_pstrq(const char *string, json_pfs *pfs)
 	    pfunc('t', parg);
 	    break;
 	default:
-	    if ((*string < ' ' && *string > 0) || *string == 0x7f) {
+	    if (((*string < ' ') && (*string > 0)) || (*string == 0x7f)) {
 		sprintf(buf, "\\u%04x", *string);
 		json_pstr(buf, pfs);
 	    } else if (*string < 0) {
@@ -1927,8 +2121,8 @@ json_pstrq(const char *string, json_pfs *pfs)
 			uc = c;
 		    }
 		} else if (c < 0xf0) {
-		    if ((string[1] & 0xc0) == 0x80 &&
-			(string[2] & 0xc0) == 0x80) {
+		    if (((string[1] & 0xc0) == 0x80) &&
+			((string[2] & 0xc0) == 0x80)) {
 			uc = ((c & 0x0f) << 12) |
 			     ((string[1] & 0x3f) << 6) | (string[2] & 0x3f);
 			string += 2;
@@ -1936,9 +2130,9 @@ json_pstrq(const char *string, json_pfs *pfs)
 			uc = c;
 		    }
 		} else if (c < 0xf8) {
-		    if ((string[1] & 0xc0) == 0x80 &&
-			(string[2] & 0xc0) == 0x80 &&
-			(string[3] & 0xc0) == 0x80) {
+		    if (((string[1] & 0xc0) == 0x80) &&
+			((string[2] & 0xc0) == 0x80) &&
+			((string[3] & 0xc0) == 0x80)) {
 			uc = ((c & 0x03) << 18) |
 			     ((string[1] & 0x3f) << 12) |
 			     ((string[2] & 0x3f) << 6) |
@@ -1948,10 +2142,10 @@ json_pstrq(const char *string, json_pfs *pfs)
 			uc = c;
 		    }
 		} else if (c < 0xfc) {
-		    if ((string[1] & 0xc0) == 0x80 &&
-			(string[2] & 0xc0) == 0x80 &&
-			(string[3] & 0xc0) == 0x80 &&
-			(string[4] & 0xc0) == 0x80) {
+		    if (((string[1] & 0xc0) == 0x80) &&
+			((string[2] & 0xc0) == 0x80) &&
+			((string[3] & 0xc0) == 0x80) &&
+			((string[4] & 0xc0) == 0x80)) {
 			uc = ((c & 0x01) << 24) |
 			     ((string[1] & 0x3f) << 18) |
 			     ((string[2] & 0x3f) << 12) |
@@ -1987,6 +2181,29 @@ json_pstrq(const char *string, json_pfs *pfs)
     pfunc('"', parg);
 }
 
+/**
+ * Conditionally quote and write string using JSON output function
+ * @param string string to be written
+ * @param pfs JSON output function
+ */
+
+static void
+json_pstrc(const char *string, json_pfs *pfs)
+{
+    if (*string && strchr(".0123456789-+", *string)) {
+	json_pstr(string, pfs);
+    } else {
+	json_pstrq(string, pfs);
+    }
+}
+
+/**
+ * Write a blob as base64 string using JSON output function
+ * @param blk pointer to blob
+ * @param len length of blob
+ * @param pfs JSON output function
+ */
+
 static void
 json_pb64(const unsigned char *blk, int len, json_pfs *pfs)
 {
@@ -2031,6 +2248,15 @@ json_pb64(const unsigned char *blk, int len, json_pfs *pfs)
     pfunc('"', parg);
 }
 
+/**
+ * Execute SQL and write output as JSON
+ * @param db SQLite database pointer
+ * @param sql SQL text
+ * @param pfunc JSON output function
+ * @param parg argument for output function
+ * @result SQLite error code
+ */
+
 static int
 json_output(sqlite3 *db, char *sql, impexp_putc pfunc, void *parg)
 {
@@ -2040,39 +2266,40 @@ json_output(sqlite3 *db, char *sql, impexp_putc pfunc, void *parg)
 
     pfs->pfunc = pfunc;
     pfs->parg = parg;
-    json_pstr("{sql:", pfs);
+    json_pstr("{\"sql\":", pfs);
     json_pstrq(sql, pfs);
-    json_pstr(",results:[", pfs);
+    json_pstr(",\"results\":[", pfs);
     do {
 	sqlite3_stmt *stmt;
 	int firstrow = 1, nrows = 0;
 	char buf[256];
 
 	++nresults;
-	json_pstr(nresults == 1 ? "{" : ",{", pfs);
+	json_pstr((nresults == 1) ? "{" : ",{", pfs);
 	result = sqlite3_prepare(db, tail, -1, &stmt, &tail);
 	if (result != SQLITE_OK) {
 doerr:
 	    if (nrows == 0) {
-		json_pstr("columns:null,rows:null,changes:0,"
-			  "last_insert_rowid:null,", pfs);
+		json_pstr("\"columns\":null,\"rows\":null,\"changes\":0,"
+			  "\"last_insert_rowid\":null,", pfs);
 	    }
-	    json_pstr("error:", pfs);
+	    json_pstr("\"error:\"", pfs);
 	    json_pstrq(sqlite3_errmsg(db), pfs);
 	    pfunc('}', parg);
 	    break;
 	}
 	result = sqlite3_step(stmt);
-	while (result == SQLITE_ROW || result == SQLITE_DONE) {
+	while ((result == SQLITE_ROW) || (result == SQLITE_DONE)) {
 	    if (firstrow) {
 		for (i = 0; i < sqlite3_column_count(stmt); i++) {
 		    char *type;
-		    json_pstr(i == 0 ? "columns:[" : ",", pfs);
-		    json_pstr("{name:", pfs);
+
+		    json_pstr((i == 0) ? "\"columns\":[" : ",", pfs);
+		    json_pstr("{\"name\":", pfs);
 		    json_pstrq(sqlite3_column_name(stmt, i), pfs);
-		    json_pstr(",decltype:", pfs);
+		    json_pstr(",\"decltype\":", pfs);
 		    json_pstrq(sqlite3_column_decltype(stmt, i), pfs);
-		    json_pstr(",type:", pfs);
+		    json_pstr(",\"type\":", pfs);
 		    switch (sqlite3_column_type(stmt, i)) {
 		    case SQLITE_INTEGER:
 			type = "integer";
@@ -2105,14 +2332,16 @@ doerr:
 		break;
 	    }
 	    ++nrows;
-	    json_pstr(nrows == 1 ? ",rows:[" : ",", pfs);
+	    json_pstr((nrows == 1) ? ",\"rows\":[" : ",", pfs);
 	    for (i = 0; i < sqlite3_column_count(stmt); i++) {
-		pfunc(i == 0 ? '[' : ',', parg);
+		pfunc((i == 0) ? '[' : ',', parg);
 		switch (sqlite3_column_type(stmt, i)) {
 		case SQLITE_INTEGER:
-		case SQLITE_FLOAT:
 		    json_pstr((char *) sqlite3_column_text(stmt, i), pfs);
 		    break;
+		case SQLITE_FLOAT:
+		    json_pstrc((char *) sqlite3_column_text(stmt, i), pfs);
+		    break;
 		case SQLITE_BLOB:
 		    json_pb64((unsigned char *) sqlite3_column_blob(stmt, i),
 			      sqlite3_column_bytes(stmt, i), pfs);
@@ -2126,7 +2355,7 @@ doerr:
 		    break;
 		}
 	    }
-	    json_pstr(i == 0 ? "null]" : "]", pfs);
+	    json_pstr((i == 0) ? "null]" : "]", pfs);
 	    result = sqlite3_step(stmt);
 	}
 	if (nrows > 0) {
@@ -2137,9 +2366,9 @@ doerr:
 	    if (nrows > 0) {
 		sprintf(buf,
 #ifdef _WIN32
-			",changes:%d,last_insert_rowid:%I64d",
+			",\"changes\":%d,\"last_insert_rowid\":%I64d",
 #else
-			",changes:%d,last_insert_rowid:%lld",
+			",\"changes\":%d,\"last_insert_rowid\":%lld",
 #endif
 			sqlite3_changes(db),
 			sqlite3_last_insert_rowid(db));
@@ -2148,23 +2377,30 @@ doerr:
 	    goto doerr;
 	}
 	if (nrows == 0) {
-	    json_pstr("columns:null,rows:null", pfs);
+	    json_pstr("\"columns\":null,\"rows\":null", pfs);
 	}
 	sprintf(buf,
 #ifdef _WIN32
-		",changes:%d,last_insert_rowid:%I64d",
+		",\"changes\":%d,\"last_insert_rowid\":%I64d",
 #else
-		",changes:%d,last_insert_rowid:%lld",
+		",\"changes\":%d,\"last_insert_rowid\":%lld",
 #endif
 		sqlite3_changes(db),
 		sqlite3_last_insert_rowid(db));
 	json_pstr(buf, pfs);
-	json_pstr(",error:null}", pfs);
+	json_pstr(",\"error\":null}", pfs);
     } while (tail && *tail);
     json_pstr("]}", pfs);
     return result;
 }
 
+/**
+ * SQLite function for JSON output, see impexp_export_json
+ * @param ctx SQLite function context
+ * @param nargs number of arguments
+ * @param args argument vector
+ */
+
 static void
 export_json_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 {
@@ -2216,6 +2452,8 @@ done:
     sqlite3_result_int(ctx, result);
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_export_json(sqlite3 *db, char *sql, impexp_putc pfunc,
 		   void *parg)
@@ -2223,6 +2461,14 @@ impexp_export_json(sqlite3 *db, char *sql, impexp_putc pfunc,
     return json_output(db, sql, pfunc, parg);
 }
 
+/**
+ * Initializer for SQLite extension load mechanism.
+ * @param db SQLite database pointer
+ * @param errmsg pointer receiving error message
+ * @param api SQLite API routines
+ * @result SQLite error code
+ */
+
 #ifdef STANDALONE
 static int
 #else
@@ -2269,6 +2515,8 @@ sqlite3_extension_init(sqlite3 *db, char **errmsg,
     return rc;
 }
 
+/* see doc in impexp.h */
+
 int
 impexp_init(sqlite3 *db)
 {
